@@ -1,9 +1,11 @@
 import os
 import io
+from io import BytesIO
+from types import NoneType
 import typing
+from typing import Type
 import numpy as np
 import cv2
-import numpy as np
 import pdf2image
 import time
 import pytesseract
@@ -11,7 +13,6 @@ import yaml
 import json
 import difflib
 import unidecode
-import matplotlib.pyplot as plt
 import PIL
 
 # print("Hello sheet music")
@@ -56,7 +57,6 @@ def textRecognizer(imagePath):
 	cv2.waitKey(0)
 
 def cropImage(img):
-	# findContentBox(img)
 	return img
 	return img[0:len(img)//2, 0:len(img[0])//2]
 
@@ -233,73 +233,6 @@ def predictParts(detectionData, instruments, imageWidth, imageHeight):
 			# Its probably a full score
 			return ["full score"], [["full score"]]
 
-# define a function which returns an image as numpy array from figure
-def get_img_from_pyplot_fig(dpi=180):
-    buf = io.BytesIO()
-    plt.savefig(buf, format="png", dpi=dpi)
-    buf.seek(0)
-    img_arr = np.frombuffer(buf.getvalue(), dtype=np.uint8)
-    buf.close()
-    img = cv2.imdecode(img_arr, 1)
-    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
-    return img
-
-# Accepts an image matrix
-# Finds a bounding box for where the main content of the page most likely is
-def findContentBox(img):
-	# print("yo")
-	# print("hmm")
-	# print("test: ", test)
-	# img = np.array([[[255, 255, 255], [2, 3, 4]], [[255, 0, 0], [4, 5, 6]]])
-	img = np.average(img, axis=2)       # Convert to black-white
-	img = 255 - img
-	rows = np.average(img, axis=1)
-	cols = np.average(img, axis=0)
-	# x_min = np.percentile(img, 95, axis=1)  # Find 95th percentile of data
-	# x_max = np.percentile(img, 5, axis=1)  # Find 5th percentile of data
-	rows_hist, _ = np.histogram(np.arange(rows.shape[0]), weights=rows, bins=16, density=False)
-	cols_hist, _ = np.histogram(np.arange(cols.shape[0]), weights=cols, bins=32, density=False)
-	# img[:,:] = img[:,:,:]
-	print("img:\n", img)
-	# print("x:", x_min, x_max)
-	print("rows:", rows)
-	print("cols:", cols)
-	print("rows_hist:\n", rows_hist)
-	print("cols_hist:\n", cols_hist)
-	# print(img[2000, 1000, 1])
-	print("shape:", img.shape)
-
-	plt.clf()
-	plt.bar(np.arange(rows_hist.shape[0]), rows_hist)
-	histogram = get_img_from_pyplot_fig()
-	# buf = io.BytesIO()
-	# plt.savefig(buf, format="png", dpi=100)
-	# buf.seek(0)
-	# histogram = np.frombuffer(buf.getvalue(), dtype=np.uint8)
-	# print("hist 1:", histogram, histogram.shape[0])
-	# histogram = cv2.imdecode(histogram, cv2.IMREAD_UNCHANGED)
-	# buf.close()
-	# print("hist 2:", histogram)
-	# histogram = cv2.cvtColor(histogram, cv2.COLOR_BGR2RGB)
-	# print("hist 3:", histogram)
-
-	kernel = np.ones((200, 200), np.float32)/(200 * 200)
-	# img_filtered = cv2.filter2D(img, -1, kernel)
-	kernel_size = (np.array((img.shape[1], img.shape[0])) //6) * 2 + 1
-	img_filtered = cv2.GaussianBlur(img, kernel_size, 0)
-	img_filtered_rows = np.average(img_filtered, axis=1)
-	img_filtered_cols = np.average(img_filtered, axis=0)
-
-	plt.clf()
-	plt.plot(img_filtered_rows)
-	img_filtered_chart = get_img_from_pyplot_fig()
-
-	x = 0
-	y = 0
-	width = 0
-	height = 0
-
-	return x, y, width, height, histogram, 255 - 4 * img_filtered, img_filtered_chart
 
 def predict_parts_in_img(img : io.BytesIO | bytes | PIL.Image.Image, instruments, use_lstm=False, tessdata_dir=None) -> typing.Tuple[list, list]:
 	if type(img) is PIL.Image.Image:
@@ -316,45 +249,29 @@ def predict_parts_in_img(img : io.BytesIO | bytes | PIL.Image.Image, instruments
 
 	return predictParts(detection_data, instruments, *img.size)
 
-def processUploadedPdf(pdfPath, imagesDirPath, instruments_file=None, instruments=None, use_lstm=False, tessdata_dir=None):
+
+def predict_parts_in_imgs(imgs, instruments, use_lstm=False, tessdata_dir=None):
 	"""
 	Arguments:
-	pdfPath                     - Full path to PDF file
-	imagesDirPath               - Full path to output images
-	instruments_file (optional) - Full path to instruments file. Accepted formats: YAML (.yaml, .yml), JSON (.json)
-	instruments      (optional) - Dictionary of instruments. Will override any provided instruments file.
-	- If neither instruments_file nor instruments is provided a default instruments file will be used.
-	use_lstm         (optional) - Use LSTM instead of legacy engine mode.
-	tessdata_dir     (optional) - Full path to tessdata directory. If not provided, whatever the environment variable TESSDATA_DIR will be used.
+	- imgs                    - a list of images representing one page each in a pdf
+	- instruments             - a dictionary of instruments
+	- use_lstm=False          - to use LSTM engine mode instead of legacy
+	- tessdata_dir=None       - full path to tessdata_dir
 
 	Returns:
-	parts                       - A list of dictionaries { "name": "[name]", "fromPage": i, "toPage": j } describing each part
-	instrumentsDefaultParts     - A dictionary { ..., "instrument_i": j, ... }, where j is the index in the parts list for the default part for instrument_i.
+	- parts                   - a list of dictionaries { "name": "[name]", "fromPage": i, "toPage": j } describing each part
+	- instrumentsDefaultParts - a dictionary { ..., "instrument_i": j, ... }, where j is the index in the parts list for the default part for instrument_i.
 	"""
-	if instruments == None:
-		if instruments_file == None:
-			with open(os.path.join(os.path.dirname(__file__), "instruments.yaml")) as file:
-				instruments = yaml.safe_load(file)
-		else:
-			with open(instruments_file) as file:
-				_, file_extension = os.path.splitext(instruments_file)
-				if file_extension.lower() in [".yaml", ".yml"]:
-					instruments = yaml.safe_load(file)
-				elif file_extension.lower() in [".json"]:
-					instruments = json.load(file)
-				else:
-					raise Exception("Instruments file with extension {} is not supported".format(file_extension))
 	parts = []
 	instrumentsDefaultParts = { instrument: None for instrument in instruments }
 	instrumentsDefaultParts["full score"] = None
-	imagePaths = generateImagesFromPdf(pdfPath, imagesDirPath, 1, None)
 	lastPartName = ""
 	lastPartNamePage = 0
 	lastInstruments = []
-	for i in range(len(imagePaths)):
-		print("side", i+1, "av", len(imagePaths))
+	for i in range(len(imgs)):
+		print("side", i+1, "av", len(imgs))
 		print("cropper...")
-		img = cropImage(cv2.imread(imagePaths[i]))
+		img = cropImage(imgs[i])
 		print("detecter...")
 		config = "--user-words sheetmusicUploader/instrumentsToLookFor.txt --psm 11 --dpi 96 -l eng"
 		if use_lstm: config += " --oem 1"
@@ -381,10 +298,47 @@ def processUploadedPdf(pdfPath, imagesDirPath, instruments_file=None, instrument
 		parts.append({
 			"name": lastPartName,
 			"fromPage": lastPartNamePage,
-			"toPage": len(imagePaths)
+			"toPage": len(imgs)
 		})
 		for k in range(len(lastInstruments)):
 			if instrumentsDefaultParts[lastInstruments[k]] == None:
 				instrumentsDefaultParts[lastInstruments[k]] = len(parts)-1
 	return parts, instrumentsDefaultParts
 
+
+def get_instruments_dict_from_file_stream(file, format="yaml"):
+	match format:
+		case "yaml":
+			return yaml.safe_load(file)
+		case "json":
+			return json.load(file)
+		case _:
+			raise Exception(f"Instruments file format {format} is not supported. Supported formats are yaml, json")
+
+
+def get_instruments_dict_from_file_path(instruments_file_path):
+	with open(instruments_file_path) as file:
+		_, file_extension = os.path.splitext(instruments_file_path)
+		if file_extension.lower() in [".yaml", ".yml"]:
+			return get_instruments_dict_from_file_stream(file)
+		elif file_extension.lower() in [".json"]:
+			return get_instruments_dict_from_file_stream(file, "json")
+		else:
+			raise Exception(f"Instruments file with extension {file_extension} is not supported. Supported extensions are .yaml, .yml, .json")
+
+def get_instruments_dict(
+	instruments_file : Type[None] | str | BytesIO | bytes = None,
+	instruments_file_format="yaml",
+	instruments=None
+	):
+	if instruments is not None:
+		return instruments
+	match instruments_file:
+		case NoneType():
+			return get_instruments_dict_from_file_path(os.path.join(os.path.dirname(__file__), "instruments.yaml"))
+		case str():
+			return get_instruments_dict_from_file_path(instruments_file)
+		case BytesIO():
+			return get_instruments_dict_from_file_stream(instruments_file, format=instruments_file_format)
+		case bytes():
+			return get_instruments_dict_from_file_stream(BytesIO(instruments_file), format=instruments_file_format)
